/** 
 *   FIXME ....
 *
 *   (c) Copyright 2004 Federico Bareilles <fede@iar.unlp.edu.ar>,
 *   Instituto Argentino de Radio Astronomia (IAR).
 *
 *   This program is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU General Public License as
 *   published by the Free Software Foundation; either version 2 of
 *   the License, or (at your option) any later version.
 *     
 *   The author does NOT admit liability nor provide warranty for any
 *   of this software. This material is provided "AS-IS" in the hope
 *   that it may be useful for others.
 *
 *   TODO: ???
 *
 *   27/12/2004: fix return val in ibus_read();
 *               add select() to write in ibus_send();
 **/

#define _GNU_SOURCE

#include <stdio.h>
#include <unistd.h>
#include <stdlib.h>
#include "iar_bus.h"

/******************************************************************************
 * Functions for de divece side 
 */

static int ibus_send( int fd, int addr_to, int action, void *data, int len)
{
	struct ib_packet_head ph;
	fd_set wfds;
	struct timeval tv;
	int fd_max;
	int retval;

	ph.addr = addr_to;
	ph.cmd = action;
	ph.size = len;
	fd_max = fd + 1;

	do {
	        FD_ZERO(&wfds);
		FD_SET(fd, &wfds);
		tv.tv_sec = 1;
		tv.tv_usec = 0;
		retval = select(fd_max, NULL, &wfds, NULL, &tv);
	}while (retval == 0);

	if ( write(fd, &ph, sizeof(struct ib_packet_head)) < 0  )
		return -1;
	if ( len > 0 ){
		if (write(fd, data, len) < 0 )
			return -1;
	}

	return 0;
}


int ibus_send_data( int fd, void *data, int len)
{
	return ibus_send( fd, IB_BROADCAST_ADDR, IB_CM_MESSAGE, data, len);
}


int ibus_send_data_to( int fd, int addr, void *data, int len)
{
	return ibus_send( fd, addr, IB_CM_MESSAGE_TO, data, len);
}


int ibus_read(int fd, struct ib_packet *d)
{
	int ret;

	ret = read(fd, d, sizeof(struct ib_packet_head) );
	if( d->h.size ){
		ret += read(fd, d->data, d->h.size );
	}

	return ret;
}


int ibus_on( int fd )
{
	struct ib_packet d;
	int ret = ibus_send( fd, IB_BUS_ADDR, IB_CM_ON, 0, 0);

	do {
		ibus_read(fd, &d);
	} while(d.h.cmd != IB_CM_ON || d.h.addr != IB_BUS_ADDR);
	
	return ret;
}


int ibus_off( int fd )
{
	struct ib_packet d;
	int ret = ibus_send( fd, IB_BUS_ADDR, IB_CM_OFF, 0, 0);

	do {
		ibus_read(fd, &d);
	} while(d.h.cmd != IB_CM_OFF || d.h.addr != IB_BUS_ADDR);

	return ret;
}


int ibus_remove( int fd )
{
	struct ib_packet d;
	int ret = ibus_send( fd, IB_BUS_ADDR, IB_CM_REMOVE, 0, 0);

	do {
		ibus_read(fd, &d);
	} while(d.h.cmd != IB_CM_REMOVE || d.h.addr != IB_BUS_ADDR);

	return ret;
}


int ibus_set_addr(int fd, int addr)
{
	struct ib_packet d;

	ibus_send( fd, IB_BUS_ADDR, 
		   IB_CM_SET_ADDR, &addr, sizeof(int));
	
	do {
		ibus_read(fd, &d);
	} while(d.h.cmd != IB_CM_SET_ADDR || d.h.addr != IB_BUS_ADDR);

	addr = ((int *)d.data)[0];

	return addr;
}


int ibus_rw(int fd, int new_state)
{
	struct ib_packet d;

	ibus_send( fd, IB_BUS_ADDR, IB_CM_RW, &new_state, sizeof(int));
	do {
		ibus_read(fd, &d);
	} while(d.h.cmd != IB_CM_RW || d.h.addr != IB_BUS_ADDR);
	new_state = ((int *)d.data)[0];

	return new_state;
}


/* End */
